print("------------ Start Input Data Engineering ------------")
# --------------------------------- Input Data Engineering ---------------------------------
# -------------------- Libraries --------------------
import pandas as pd
import datetime as dt
import numpy as np
# -------------------- Collect Target Data --------------------
target_df = pd.read_csv('./Input_Data/target.csv')
target_df['Month'] = pd.to_datetime(target_df['timestamp']).dt.month
target_df['Day'] = pd.to_datetime(target_df['timestamp']).dt.day
target_df['Hour'] = pd.to_datetime(target_df['timestamp']).dt.hour
general_target_df = target_df.groupby(['Month','Day','Hour']).mean().reset_index().round(0)
# -------------------- Collect Weather Data --------------------
weather_df = pd.read_csv('./Input_Data/Weather.csv')
weather_df['Month'] = pd.to_datetime(weather_df['timestamp']).dt.month
weather_df['Day'] = pd.to_datetime(weather_df['timestamp']).dt.day
weather_df['Hour'] = pd.to_datetime(weather_df['timestamp']).dt.hour
general_weather_df = weather_df.groupby(['Month','Day','Hour']).mean().reset_index()
# -------------------- Collect Public Holidays --------------------
publicHolidays_df = pd.read_csv('./Input_Data/publicHolidays.csv')
publicHolidays_df['Year'] = pd.to_datetime(publicHolidays_df['date']).dt.year
publicHolidays_df['Month'] = pd.to_datetime(publicHolidays_df['date']).dt.month
publicHolidays_df['Day'] = pd.to_datetime(publicHolidays_df['date']).dt.day
# -------------------- Generate Simulated Timestamp --------------------
now_dt = dt.datetime.now()
end_dt = (now_dt+dt.timedelta(days=2))
start_dt = (end_dt-dt.timedelta(days=365)).date()
timestamp_df = pd.DataFrame(pd.date_range(start = start_dt, end = end_dt, freq='H'),columns=['datetime'])
timestamp_df['Year'] = pd.to_datetime(timestamp_df['datetime']).dt.year
timestamp_df['Month'] = pd.to_datetime(timestamp_df['datetime']).dt.month
timestamp_df['Day'] = pd.to_datetime(timestamp_df['datetime']).dt.day
timestamp_df['Hour'] = pd.to_datetime(timestamp_df['datetime']).dt.hour
# -------------------- Combine Data & Generate Tim Input --------------------
tim_input_df = timestamp_df.merge(general_target_df,on=['Month','Day','Hour'],how='left').merge(general_weather_df,on=['Month','Day','Hour'],how='left').merge(publicHolidays_df,on=['Year','Month','Day'],how='left').drop(columns=['date','Year','Month','Day','Hour'])
tim_input_df['quantity'] = np.where(tim_input_df['datetime']>now_dt,np.nan,tim_input_df['quantity'])
timestamp = 'datetime'
target_variable = 'quantity'
predictor_candidates = [s for s in list(tim_input_df.columns) if s not in [timestamp,target_variable]]
tim_input_df = tim_input_df[[timestamp,target_variable]+predictor_candidates].reset_index(drop=True)
tim_input_df.to_csv('./Input_Data/tim_input_df.csv',index=False)
# --------------------------------- END OF SCRIPT ---------------------------------
print("------------ End Input Data Engineering ------------")
