print("------------ Start TIM Forecasting ------------")
# ------------------------------------------------- Autorun Template -------------------------------------------------
# ------------------------------------------------- 0. Setup -------------------------------------------------
import pandas as pd
import datetime as dt
import json
import os
pd.set_option('mode.chained_assignment', None)
from tim import Tim
tim_credentials = json.load(open('credentials.json'))
client = Tim(email=tim_credentials['email'],password=tim_credentials['password'])
# ------------------------------------------------- 1. Data Preparation -------------------------------------------------
tim_input_df = pd.read_csv('./Input_Data/tim_input_df.csv')
# ------------------------------------------------- 2. TIM Setup -------------------------------------------------
job_configuration = json.load(open('TIM_job_configuration.json'))
# ------------------------------------------------- Extra Functions -------------------------------------------------
def get_forecast_job_results_accuracy(response):   
    bin_json = response['bin']
    bin_accuracy_df = pd.DataFrame()
    for n in bin_json:
        bin_accuracy_df = bin_accuracy_df.append(pd.DataFrame(n).reset_index().rename(columns={'index':'KPI'}))
    bin_accuracy_df['accuracy_type'] = 'bin'

    samplesAhead_json = response['samplesAhead']
    samplesAhead_accuracy_df = pd.DataFrame()
    for n in samplesAhead_json:
        samplesAhead_accuracy_df = samplesAhead_accuracy_df.append(pd.DataFrame(n).reset_index().rename(columns={'index':'KPI'}))
    samplesAhead_accuracy_df['accuracy_type'] = 'samplesAhead'

    all_accuracy_df = pd.DataFrame(response['all']).reset_index().rename(columns={'index':'KPI'})
    all_accuracy_df['accuracy_type'] = 'all'
    id_columns = ['KPI','name','accuracy_type']
    acc_df = all_accuracy_df.append(samplesAhead_accuracy_df).append(bin_accuracy_df)
    df = pd.melt(acc_df, id_vars=id_columns, value_vars=list(set(acc_df.columns)-set(id_columns)))
    return df

def get_forecast_job_results_model(response):
    properties = response['model']['modelZoo']['variableProperties']
    models = response['model']['modelZoo']['models']
    
    pi_df = pd.DataFrame(properties).sort_values(by='importance',ascending=False)
    pi_df['rel_importance'] = pi_df['importance']/pi_df.sum()['importance']
    
    features = []
    for m in models:
        terms = m['terms']
        for count,t in enumerate(terms):
            f,b = find_feature(t['parts'])
            features.append([m['index'],count,f,t['importance'],b])
    fi_df = pd.DataFrame(features,columns=['Model','Term','Feature','importance','beta'])
    return pi_df,fi_df,models

def find_feature(sub_parts):
    dow_list = ['Monday','Tuesday','Wednesday','Thursday','Friday','Saturday','Sunday']
    month_list = ['January','February','March','April','May','June','July','August','September','October','November','December']
    features_list = []
    for c,s in enumerate(sub_parts):
        if   s['type']=='β':
            sub_feature = ''
        elif s['type']=='TimeOffsets':
            sub_feature = s['predictor']+'(t'+str(s['offset'])+')'
        elif s['type']=='RestOfWeek':
            sub_feature ='DoW(t'+str(s['offset'])+') <= '+dow_list[s['day']-1]
        elif s['type']=='Intercept':
            sub_feature = 'Intercept('+str(int(s['value']))+')'
        elif s['type']=='Cos':
            sub_feature = 'Cos('+str(int(s['period']))+';'+s['unit']+')'
        elif s['type']=='Sin':
            sub_feature = 'Sin('+str(int(s['period']))+';'+s['unit']+')'
        elif s['type']=='ExponentialMovingAverage':
            sub_feature = 'EMA_'+s['predictor']+'(t'+str(int(s['offset']))+'; w='+str(int(s['window']))+')'
        elif s['type']=='Identity':
            sub_feature = s['predictor']
        elif s['type']=='PiecewiseLinear':
            sub_feature = 'max(0;'+str(s['subtype'])+'*('+str(round(s['knot'],6))+'-'+s['predictor']+'(t'+str(s['offset'])+')))'
        elif s['type']=='SimpleMovingAverage':
            sub_feature = 'SMA_'+s['predictor']+'(t'+str(int(s['offset']))+'; w='+str(int(s['window']))+')'
        elif s['type']=='Fourier':
            sub_feature = 'Fourier('+str(s['period'])+')'
        elif s['type']=='Weekday':
            sub_feature = 'DoW(t'+str(s['offset'])+') = '+dow_list[s['day']-1]
        elif s['type']=='Month':
            sub_feature = 'Month<='+month_list[s['month']]
        elif s['type']=='PublicHoliday':
            sub_feature = s['predictor']
        elif s['type']=='Trend':
            sub_feature = 'Trend'
        else:
            sub_feature = '_test_'
        if s['type']=='β':
            features_list.append(sub_feature)
            beta = s['value']
        else:
            features_list.append(' & '+sub_feature) if c>0 else features_list.append(sub_feature)
    feature_output = ''.join(str(e) for e in features_list)
    return feature_output,beta
# ------------------------------------------------- 3. API Call -------------------------------------------------
forecast_response = client.clean_forecast(dataset = tim_input_df, job_configuration = job_configuration, handle_dataset_upload_status_poll = print, handle_forecast_status_poll = print)
# ------------------------------------------------- 4. Collect Results -------------------------------------------------
job_meta_data = forecast_response[0]
properties_df,features_df,model = get_forecast_job_results_model(forecast_response[1])
forecast_job_results_table_df = forecast_response[2]
for i in forecast_job_results_table_df['forecast_type'].unique():
    forecast_job_results_table_df.loc[forecast_job_results_table_df['forecast_type']==i, i] = forecast_job_results_table_df['forecast']
accuracy_df = get_forecast_job_results_accuracy(forecast_response[3])
job_logs_df = pd.DataFrame(forecast_response[4]).sort_values(by='createdAt').reset_index(drop=True)
predictions_df = forecast_job_results_table_df[forecast_job_results_table_df['forecast_type']=='Production']
# ------------------------------------------------- 5. Save Results -------------------------------------------------
now_dt = dt.datetime.now().strftime("%Y_%m_%d_%H_%M_%S")
date_dir = './Output_Data/'+str(now_dt)+'/'
try:
    os.mkdir(date_dir)
except OSError:
    print ("Creation of the directory %s failed" % date_dir)
else:
    print ("Successfully created the directory %s " % date_dir)
forecast_job_results_table_df.to_csv(date_dir+'forecast_job_results_table_df.csv',index=False)
accuracy_df.to_csv(date_dir+'accuracy_df.csv',index=False)
properties_df.to_csv(date_dir+'properties_df.csv',index=False)
features_df.to_csv(date_dir+'features_df.csv',index=False)
job_logs_df.to_csv(date_dir+'job_logs_df.csv',index=False)
predictions_df.to_csv(date_dir+'predictions_df.csv',index=False)
tim_input_df.to_csv(date_dir+'tim_input_df.csv',index=False)
# ------------------------------------------------- 6. Save Latests Results -------------------------------------------------
forecast_job_results_table_df.to_csv('./Results/'+'forecast_job_results_table_df.csv',index=False)
accuracy_df.to_csv('./Results/'+'accuracy_df.csv',index=False)
properties_df.to_csv('./Results/'+'properties_df.csv',index=False)
features_df.to_csv('./Results/'+'features_df.csv',index=False)
job_logs_df.to_csv('./Results/'+'job_logs_df.csv',index=False)
predictions_df.to_csv('./Results/'+'predictions_df.csv',index=False)
tim_input_df.to_csv('./Results/'+'tim_input_df.csv',index=False)
# --------------------------------- END OF SCRIPT ---------------------------------
print("------------ End TIM Forecasting ------------")
